<?php
/*
Plugin Name: Where did that JS/CSS come from?
Plugin URI: http://w-shadow.com/blog/2010/12/14/where-did-that-js-css-come-from/
Description: Displays a list of JS/CSS files used on the current page, and which PHP script(s) added them. The list is added to the page footer and is only visible to users who have the "activate_plugins" privilege.  
Version: 1.0
Author: Janis Elsts
Author URI: http://w-shadow.com/blog/
*/

class wsSourceDetector {
	private $patches = array(
		array(
			'file' => 'wp-includes/functions.wp-scripts.php',
			'find' => 'function wp_enqueue_script( $handle, $src = false, $deps = array(), $ver = false, $in_footer = false ) {',
			'insert' => ' /*<WS>*/ do_action(\'wp_enqueue_script\', $handle, $src, $deps, $ver, $in_footer); /*</WS>*/ ',
		),
		array(
			'file' => 'wp-includes/functions.wp-styles.php',
			'find' => 'function wp_enqueue_style( $handle, $src = false, $deps = array(), $ver = false, $media = \'all\' ) {',
			'insert' => ' /*<WS>*/ do_action(\'wp_enqueue_style\', $handle, $src, $deps, $ver, $media); /*</WS>*/ ',
		),
	);
	
	private $scripts = array();
	private $styles = array();
	
	function __construct(){
		register_activation_hook(__FILE__, array($this, 'applyCorePatches'));
		register_deactivation_hook(__FILE__, array($this, 'removeCorePatches'));
		add_action('shutdown', array($this, 'displayResults'));
		
		//These are custom hooks that we add by patching the WP core (see $this->patches).
		add_action('wp_enqueue_script', array($this, 'logScript'), 10, 5);
		add_action('wp_enqueue_style', array($this, 'logStyle'), 10, 5);
	}
	
	function applyCorePatches() {
		foreach($this->patches as $patch){
			$this->applyPatch(ABSPATH . $patch['file'], $patch['find'], $patch['insert']);
		}
	}
	
	function removeCorePatches(){
		foreach($this->patches as $patch){
			$this->removePatch(ABSPATH . $patch['file'], $patch['insert']);
		}
	}
	
	function applyPatch($filename, $find, $insert){
		$contents = file_get_contents($filename);
		
		$found = strpos($contents, $insert);
		if ( $found !== false ){
			return true;
		}
		
		$contents = str_replace($find, $find . $insert, $contents);
		file_put_contents($filename, $contents);
		
		return true;
	}
	
	function removePatch($filename, $remove){
		$contents = file_get_contents($filename);
		$contents = str_replace($remove, '', $contents);
		file_put_contents($filename, $contents);
		return true;
	}
	
	function logScript($handle, $src, $deps, $ver, $in_footer){
		list($file, $line) = $this->findFunctionCall(debug_backtrace(), 'wp_enqueue_script');
		$this->scripts[$handle][] = compact('handle', 'src', 'deps', 'ver', 'in_footer', 'line', 'file');
	}
	
	function logStyle($handle, $src, $deps, $ver, $media){
		list($file, $line) = $this->findFunctionCall(debug_backtrace(), 'wp_enqueue_style');
		$this->styles[$handle][] = compact('handle', 'src', 'deps', 'ver', 'media', 'line', 'file');
	}
	
	function findFunctionCall($backtrace, $functionName){
		foreach($backtrace as $call){
			if ( array_key_exists('function', $call) && ( $call['function'] == $functionName ) ){
				return array(
					array_key_exists('file', $call) ? $call['file'] : '',
					array_key_exists('line', $call) ? $call['line'] : 0
				);				
			}
		}
		return array('', 0);
	}
	
	function displayResults(){
		if ( !current_user_can('activate_plugins') || ( defined('DOING_AJAX') && DOING_AJAX ) ){
			return;
		}
		?>
		<style>
			#ws_source_detector {
				background: white;
				color: black;
				margin: 1em 2em 2em 2em;
				padding: 1em;
				border: 1px solid gray;
				border-radius: 12px;
				font-size: 11pt;
				font-family: "Verdana", "Tahoma", "Helvetica", sans-serif;
				text-align: left;
			}
			
			#ws_source_detector h2 {
				margin: 0.4em 0 0.5em 0;
				font-size: 1.2em;
			}
			
			#ws_source_detector ul {
				margin: 0.3em 0 1.8em 2em;
				text-align: left;
			}
			
			#ws_source_detector li {
				list-style: none;
			}
			
			#ws_source_detector .gray {
				color: gray;
			}
		</style>
		<div id="ws_source_detector">
			<?php
			global $wp_scripts, $wp_styles;
			$this->printResultList('Scripts', $this->scripts, $wp_scripts);
			$this->printResultList('Styles', $this->styles, $wp_styles);
			?>
		</div>
		<?php
	}
	
	function printResultList($title, $handles, $dependencies){
		printf('<h2>%s (%d)</h2>', $title, count($handles));
		if ( empty($handles) ){
			echo 'None detected.';
		} else {
			echo '<ul>';
			foreach($handles as $handle => $log){
				echo '<li>';
				
				$source = $this->getSourceFor($handle, $dependencies);
				if ( !empty($source) ){
					printf('<strong title="%s">%s</strong>', esc_attr($source), basename($source));
				} else {
					printf('<strong>%s</strong> (group)', $handle);
				}
			
				echo '<ul>';
				foreach($log as $entry){
					printf('<li>Added here: %s:%d</li>', $entry['file'], $entry['line']);
				}
				echo '</ul>';
				
				echo '</li>';
			}
			echo '</ul>';
		}
	}
	
	function getSourceFor($handle, $dependencies){
		if ( !isset($dependencies->registered[$handle]) ){
			return '';
		}
		$src = $dependencies->registered[$handle]->src;
		if ( !is_string($src) ){
			return '';
		}
		return $src;		
	}
}

$ws_source_detector = new wsSourceDetector();

